#!/bin/env python

"""
This wrapper runs the driver 3 times and collect results:
  ./driver -M 32 -N 32
  ./driver -M 64 -N 64
  ./driver -M 61 -N 67
"""


import subprocess;
import re;
import os;
import sys;

# compute the score depending on the number of cache misses
def computeMissScore(miss, lower, upper, full_score):
  if miss <= lower:
    return full_score
  if miss >= upper: 
    return 0

  score = (miss - lower) * 1.0 
  range = (upper- lower) * 1.0
  return (1 - score / range) * full_score


def main():
  stdout_data = subprocess.check_output("./driver -M 32 -N 32 | grep AUTORESULTS", shell=True)
  result32 = re.findall(r'(\d+)', stdout_data)
  print result32

  stdout_data = subprocess.check_output("./driver -M 64 -N 64 | grep AUTORESULTS", shell=True)
  result64 = re.findall(r'(\d+)', stdout_data)
  print result64

  stdout_data = subprocess.check_output("./driver -M 61 -N 67 | grep AUTORESULTS", shell=True)
  result61 = re.findall(r'(\d+)', stdout_data)
  print result61

  cachesim_results  = map(int, result32[0:6])
  trans_correctness = int(result32[6]) * int(result64[6]) * int(result61[6]);
  miss32 = int(result32[7])
  miss64 = int(result64[7])
  miss61 = int(result61[7])

  score32 = computeMissScore(miss32, 350, 1150, 8) * trans_correctness;
  score64 = computeMissScore(miss64, 1850, 4700, 8) * trans_correctness;
  score61 = computeMissScore(miss61, 2000, 4400, 10) * trans_correctness;

  print "============="
  print "cachesim scoreing: ", cachesim_results
  print "transpose_submit correctness: ", trans_correctness
  print "32*32 matrix transpose misses and score: ", miss32, "(", score32, ")" 
  print "64*64 matrix transpose misses and score: ", miss64, "(", score64, ")"
  print "61*67 matrix transpose misses abd scire: ", miss61, "(", score61, ")"
  print "============="

  autoresult="%f:%f:%f:%f:%f:%f:%d:%f:%f:%f" % (
      cachesim_results[0],cachesim_results[1],
      cachesim_results[2],cachesim_results[3],
      cachesim_results[4],cachesim_results[5],
      trans_correctness,
      score32,score64, score61)


  andrewid = os.environ['USER']

  url='"http://unofficial.fish.ics.cs.cmu.edu/unofficialSubmit.rb?course=15213-f10&user=%s&assessment=cachelab&result=%s"' % (andrewid, autoresult)
  
  if len(sys.argv) == 1:
    print "Posting unofficial score (%s) to Autolab:" % andrewid
    cmd="wget -q -O - -t 5 %s" % (url)
    stdout_data=subprocess.check_output(cmd, shell=True)
    print stdout_data

  print "AUTORESULTS=%s" % autoresult


# execute main only if called as a script
if __name__ == "__main__":
  main()

